@@ -1,5 +1,5 @@
 #
-# Copyright 2007 David Snopek <dsnopek@gmail.com>
+# Copyright 2007, 2008 David Snopek <dsnopek@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,14 +19,14 @@ package POE::Component::Server::Stomp;
 
 use POE::Session;
 use POE::Component::Server::TCP;
-use POE::Filter::Stream;
+use POE::Filter::Stomp;
 use IO::String;
 use Net::Stomp::Frame;
 use Carp qw(croak);
 use vars qw($VERSION);
 use strict;
 
-$VERSION = '0.2.2';
+$VERSION = '0.2.3';
 
 sub new
 {
@@ -59,11 +59,8 @@ sub new
 	{
 		my ($kernel, $input) = @_[ KERNEL, ARG0 ];
 
-		# create the frame.
-		my $frame = _parse_frame( $input );
-
 		# Replace ARG0 with the parsed frame.
-		splice(@_, ARG0, 1, $frame);
+		splice(@_, ARG0, 1, $input);
 
 		# pass to the user handler
 		$handle_frame->(@_);
@@ -81,7 +78,8 @@ sub new
 		ClientError        => $client_error,
 		ClientDisconnected => $client_disconnected,
 
-		ClientFilter => [ "POE::Filter::Line", Literal => "\000" ],
+		# Use Keven Esteb's awesome Stomp filter module!
+		ClientFilter => "POE::Filter::Stomp",
 
 		# pass everything left as arguments to the PoCo::Server::TCP
 		# contructor.
@@ -92,43 +90,6 @@ sub new
 	return undef;
 }
 
-sub _parse_frame
-{
-	my ($input) = @_;
-
-	my $io = IO::String->new( $input );
-
-	my $command;
-	my $headers;
-	my $body;
-
-	# read the command
-	$command = $io->getline;
-	chop $command;
-
-	# read headers
-	while (1)
-	{
-		my $line = $io->getline;
-		chop $line;
-		last if $line eq "";
-		my ( $key, $value ) = split /: ?/, $line, 2;
-		$headers->{$key} = $value;
-	}
-
-	# read the body (all the remaining data)
-	$io->read( $body, (length($input) - $io->tell()) );
-
-	# create the frame.
-	my $frame = Net::Stomp::Frame->new({
-		command => $command,
-		headers => $headers,
-		body    => $body
-	});
-
-	return $frame;
-}
-
 1;
 
 __END__
@@ -141,49 +102,49 @@ POE::Component::Server::Stomp - A generic Stomp server for POE
 
 =head1 SYNOPSIS
 
-	use POE qw(Component::Server::Stomp);
-	use Net::Stomp::Frame;
-	use strict;
+  use POE qw(Component::Server::Stomp);
+  use Net::Stomp::Frame;
+  use strict;
 
-	POE::Component::Server::Stomp->new(
-		HandleFrame        => \&handle_frame,
-		ClientDisconnected => \&client_disconnected,
-		ClientErrorr       => \&client_error
-	);
+  POE::Component::Server::Stomp->new(
+    HandleFrame        => \&handle_frame,
+    ClientDisconnected => \&client_disconnected,
+    ClientErrorr       => \&client_error
+  );
 
-	POE::Kernel->run();
-	exit;
+  POE::Kernel->run();
+  exit;
 
-	sub handle_frame
-	{
-		my ($kernel, $heap, $frame) = @_[ KERNEL, HEAP, ARG0 ];
+  sub handle_frame
+  {
+    my ($kernel, $heap, $frame) = @_[ KERNEL, HEAP, ARG0 ];
 
-		print "Recieved frame:\n";
-		print $frame->as_string() . "\n";
+    print "Recieved frame:\n";
+    print $frame->as_string() . "\n";
 
-		# allow Stomp clients to connect by playing along.
-		if ( $frame->command eq 'CONNECT' )
-		{
-			my $response = Net::Stomp::Frame->new({
-				command => 'CONNECTED'
-			});
-			$heap->{client}->put( $response->as_string . "\n" );
-		}
-	}
+    # allow Stomp clients to connect by playing along.
+    if ( $frame->command eq 'CONNECT' )
+    {
+      my $response = Net::Stomp::Frame->new({
+        command => 'CONNECTED'
+      });
+      $heap->{client}->put( $response->as_string . "\n" );
+    }
+  }
 
-	sub client_disconnected
-	{
-		my ($kernel, $heap) = @_[ KERNEL, HEAP ];
+  sub client_disconnected
+  {
+    my ($kernel, $heap) = @_[ KERNEL, HEAP ];
 
-		print "Client disconnected\n";
-	}
+    print "Client disconnected\n";
+  }
 
-	sub client_error
-	{
-		my ($kernel, $name, $number, $message) = @_[ KERNEL, ARG0, ARG1, ARG2 ];
+  sub client_error
+  {
+    my ($kernel, $name, $number, $message) = @_[ KERNEL, ARG0, ARG1, ARG2 ];
 
-		print "ERROR: $name $number $message\n";
-	}
+    print "ERROR: $name $number $message\n";
+  }
 
 =head1 DESCRIPTION
 
@@ -198,13 +159,19 @@ For a full-fledged message queue that uses this module:
 
 L<POE::Component::MessageQueue>
 
+=head1 SEE ALSO
+
+L<POE::Component::Server::TCP>,
+L<POE::Filter::Stomp>,
+L<Net::Stomp>
+
 =head1 BUGS
 
 Probably.
 
 =head1 AUTHORS
 
-Copyright 2007 David Snopek.
+Copyright 2007, 2008 David Snopek.
 
 =cut
 
